@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Guido Guenther <agx@sigxcup.org>
+ * Copyright (C) 2009 Guido Guenther <agx@sigxcpu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,20 +32,26 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <glade/glade.h>
 
 #include "krb5-auth-gconf-tools.h"
+#include "krb5-auth-tools.h"
+
+#define PKINIT_SMARTCARD "PKCS11:" SC_PKCS11
+#define PKINIT_FILE "FILE:"
 
 #define N_LISTENERS 8
 
 typedef struct {
-  GladeXML    *xml;
+  GtkBuilder  *xml;
   GConfClient *client;
 
   GtkWidget *dialog;
   GtkWidget *principal_entry;
   GtkWidget *pkuserid_entry;
+  GtkWidget *pkuserid_button;
+  GtkWidget *smartcard_toggle;
   GtkWidget *pkanchors_entry;
+  GtkWidget *pkanchors_button;
   GtkWidget *forwardable_toggle;
   GtkWidget *proxiable_toggle;
   GtkWidget *renewable_toggle;
@@ -102,7 +108,7 @@ ka_preferences_dialog_setup_principal_entry (KaPreferencesDialog *dialog)
 {
   char     *principal = NULL;
 
-  dialog->principal_entry = glade_xml_get_widget (dialog->xml, "principal_entry");
+  dialog->principal_entry = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "principal_entry"));
   g_assert (dialog->principal_entry != NULL);
 
   if (!ka_gconf_get_string (dialog->client, KA_GCONF_KEY_PRINCIPAL, &principal))
@@ -173,7 +179,7 @@ ka_preferences_dialog_setup_pkuserid_entry (KaPreferencesDialog *dialog)
 {
   char     *pkuserid = NULL;
 
-  dialog->pkuserid_entry = glade_xml_get_widget (dialog->xml, "pkuserid_entry");
+  dialog->pkuserid_entry = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "pkuserid_entry"));
   g_assert (dialog->pkuserid_entry != NULL);
 
   if (!ka_gconf_get_string (dialog->client, KA_GCONF_KEY_PK_USERID, &pkuserid))
@@ -185,7 +191,7 @@ ka_preferences_dialog_setup_pkuserid_entry (KaPreferencesDialog *dialog)
       g_free (pkuserid);
 
   g_signal_connect (dialog->pkuserid_entry, "changed",
-      G_CALLBACK (ka_preferences_dialog_pkuserid_changed), dialog);
+                    G_CALLBACK (ka_preferences_dialog_pkuserid_changed), dialog);
   if (!gconf_client_key_is_writable (dialog->client, KA_GCONF_KEY_PK_USERID, NULL)) {
       gtk_widget_set_sensitive (dialog->pkuserid_entry, FALSE);
   }
@@ -241,9 +247,9 @@ ka_preferences_dialog_pkanchors_changed (GtkEntry *entry,
 static void
 ka_preferences_dialog_setup_pkanchors_entry (KaPreferencesDialog *dialog)
 {
-  char     *pkanchors = NULL;
+  char *pkanchors = NULL;
 
-  dialog->pkanchors_entry = glade_xml_get_widget (dialog->xml, "pkanchors_entry");
+  dialog->pkanchors_entry = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "pkanchors_entry"));
   g_assert (dialog->pkanchors_entry != NULL);
 
   if (!ka_gconf_get_string (dialog->client, KA_GCONF_KEY_PK_ANCHORS, &pkanchors))
@@ -268,9 +274,154 @@ ka_preferences_dialog_setup_pkanchors_entry (KaPreferencesDialog *dialog)
 }
 
 
+static void
+ka_preferences_toggle_pkuserid_entry (gboolean state, KaPreferencesDialog *dialog)
+{
+  gtk_widget_set_sensitive (dialog->pkuserid_entry, state);
+  gtk_widget_set_sensitive (dialog->pkuserid_button, state);
+}
+
+
+static void
+ka_preferences_dialog_smartcard_toggled (GtkToggleButton *toggle,
+                                         KaPreferencesDialog *dialog)
+{
+  gboolean smartcard = gtk_toggle_button_get_active (toggle);
+  static gchar *old_path = NULL;
+
+  if (smartcard) {
+      const char *path;
+
+      path = gtk_entry_get_text (GTK_ENTRY(dialog->pkuserid_entry));
+      if (g_strcmp0 (path, PKINIT_SMARTCARD)) {
+          g_free (old_path);
+          old_path = g_strdup (path);
+      }
+      ka_preferences_toggle_pkuserid_entry (FALSE, dialog);
+      gconf_client_set_string (dialog->client, KA_GCONF_KEY_PK_USERID, PKINIT_SMARTCARD, NULL);
+  } else {
+      ka_preferences_toggle_pkuserid_entry (TRUE, dialog);
+      if (old_path)
+          gconf_client_set_string (dialog->client, KA_GCONF_KEY_PK_USERID, old_path, NULL);
+      else
+          gconf_client_unset (dialog->client, KA_GCONF_KEY_PK_USERID, NULL);
+  }
+}
+
+
+static void
+ka_preferences_dialog_setup_smartcard_toggle(KaPreferencesDialog *dialog)
+{
+  char *pkuserid = NULL;
+
+  dialog->smartcard_toggle = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "smartcard_toggle"));
+  g_assert (dialog->smartcard_toggle != NULL);
+
+  if (!ka_gconf_get_string (dialog->client, KA_GCONF_KEY_PK_USERID, &pkuserid))
+      g_warning ("Getting pkanchors failed");
+
+  g_signal_connect (dialog->smartcard_toggle, "toggled",
+              G_CALLBACK (ka_preferences_dialog_smartcard_toggled), dialog);
+
+  if (!g_strcmp0 (pkuserid, PKINIT_SMARTCARD))
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->smartcard_toggle), TRUE);
+  else
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->smartcard_toggle), FALSE);
+
+  if (pkuserid)
+      g_free (pkuserid);
+}
+
+
+static void
+ka_preferences_dialog_browse_certs (KaPreferencesDialog *dialog, GtkEntry *entry)
+{
+  GtkWidget *filechooser;
+  GtkFileFilter *cert_filter, *all_filter;
+  gchar *filename = NULL;
+  const gchar *current;
+  gint ret;
+
+  filechooser = gtk_file_chooser_dialog_new(_("Choose Certificate"),
+                                            GTK_WINDOW(dialog->dialog),
+                                            GTK_FILE_CHOOSER_ACTION_OPEN,
+                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                            NULL);
+
+  current = gtk_entry_get_text (entry);
+  if (current && g_str_has_prefix (current, PKINIT_FILE) &&
+      strlen(current) > strlen (PKINIT_FILE)) {
+      gtk_file_chooser_select_filename (GTK_FILE_CHOOSER(filechooser),
+                                        (const gchar*)&current[strlen(PKINIT_FILE)]);
+  }
+
+  cert_filter = g_object_ref_sink (gtk_file_filter_new ());
+  gtk_file_filter_add_mime_type (cert_filter, "application/x-x509-ca-cert");
+  gtk_file_filter_set_name (cert_filter, _("X509 Certificates"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filechooser), cert_filter);
+  all_filter = g_object_ref_sink (gtk_file_filter_new ());
+  gtk_file_filter_add_pattern (all_filter, "*");
+  gtk_file_filter_set_name (all_filter, _("all files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filechooser), all_filter);
+
+  ret = gtk_dialog_run (GTK_DIALOG(filechooser));
+  if (ret == GTK_RESPONSE_ACCEPT)
+     filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(filechooser));
+  gtk_widget_destroy (GTK_WIDGET(filechooser));
+
+  if (filename) {
+      gchar *cert = g_strconcat( PKINIT_FILE, filename, NULL);
+      gtk_entry_set_text (entry, cert);
+      g_free (filename);
+      g_free (cert);
+  }
+  g_object_unref (cert_filter);
+  g_object_unref (all_filter);
+}
+
+static void
+ka_preferences_dialog_browse_pkuserids (GtkButton *button G_GNUC_UNUSED,
+                                       KaPreferencesDialog *dialog)
+{
+  ka_preferences_dialog_browse_certs (dialog,
+                                      GTK_ENTRY(dialog->pkuserid_entry));
+}
+
+static void
+ka_preferences_dialog_browse_pkanchors(GtkButton *button G_GNUC_UNUSED,
+                                       KaPreferencesDialog *dialog)
+{
+  ka_preferences_dialog_browse_certs (dialog,
+                                      GTK_ENTRY(dialog->pkanchors_entry));
+}
+
+static void
+ka_preferences_dialog_setup_pkuserid_button (KaPreferencesDialog *dialog)
+{
+  dialog->pkuserid_button = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "pkuserid_button"));
+  g_assert (dialog->pkuserid_button != NULL);
+
+  g_signal_connect (dialog->pkuserid_button, "clicked",
+                    G_CALLBACK (ka_preferences_dialog_browse_pkuserids), dialog);
+
+}
+
+static void
+ka_preferences_dialog_setup_pkanchors_button (KaPreferencesDialog *dialog)
+{
+  dialog->pkanchors_button = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "pkanchors_button"));
+  g_assert (dialog->pkanchors_button != NULL);
+
+  g_signal_connect (dialog->pkanchors_button, "clicked",
+                    G_CALLBACK (ka_preferences_dialog_browse_pkanchors), dialog);
+
+}
+
+
 static void
 ka_preferences_dialog_forwardable_toggled (GtkToggleButton *toggle,
-                                     KaPreferencesDialog *dialog)
+                                           KaPreferencesDialog *dialog)
 {
   gboolean forwardable;
 
@@ -303,7 +454,7 @@ ka_preferences_dialog_setup_forwardable_toggle (KaPreferencesDialog *dialog)
 {
   gboolean forwardable;
 
-  dialog->forwardable_toggle = glade_xml_get_widget (dialog->xml, "forwardable_toggle");
+  dialog->forwardable_toggle = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "forwardable_toggle"));
   g_assert (dialog->forwardable_toggle != NULL);
 
   forwardable = gconf_client_get_bool (dialog->client, KA_GCONF_KEY_FORWARDABLE, NULL);
@@ -361,7 +512,7 @@ ka_preferences_dialog_setup_proxiable_toggle (KaPreferencesDialog *dialog)
 {
   gboolean proxiable;
 
-  dialog->proxiable_toggle = glade_xml_get_widget (dialog->xml, "proxiable_toggle");
+  dialog->proxiable_toggle = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "proxiable_toggle"));
   g_assert (dialog->proxiable_toggle != NULL);
 
   proxiable = gconf_client_get_bool (dialog->client, KA_GCONF_KEY_PROXIABLE, NULL);
@@ -419,7 +570,7 @@ ka_preferences_dialog_setup_renewable_toggle (KaPreferencesDialog *dialog)
 {
   gboolean renewable;
 
-  dialog->renewable_toggle = glade_xml_get_widget (dialog->xml, "renewable_toggle");
+  dialog->renewable_toggle = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "renewable_toggle"));
   g_assert (dialog->renewable_toggle != NULL);
 
   renewable = gconf_client_get_bool (dialog->client, KA_GCONF_KEY_RENEWABLE, NULL);
@@ -475,7 +626,7 @@ ka_preferences_dialog_setup_trayicon_toggle (KaPreferencesDialog *dialog)
 {
   gboolean trayicon;
 
-  dialog->trayicon_toggle = glade_xml_get_widget (dialog->xml, "trayicon_toggle");
+  dialog->trayicon_toggle = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "trayicon_toggle"));
   g_assert (dialog->trayicon_toggle != NULL);
 
   trayicon = gconf_client_get_bool (dialog->client, KA_GCONF_KEY_SHOW_TRAYICON, NULL);
@@ -532,7 +683,7 @@ ka_preferences_dialog_setup_prompt_mins_entry (KaPreferencesDialog *dialog)
 {
   gint prompt_mins;
 
-  dialog->prompt_mins_entry = glade_xml_get_widget (dialog->xml, "prompt_mins_entry");
+  dialog->prompt_mins_entry = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "prompt_mins_entry"));
   g_assert (dialog->prompt_mins_entry != NULL);
 
   prompt_mins = gconf_client_get_int (dialog->client, KA_GCONF_KEY_PROMPT_MINS, NULL);
@@ -561,36 +712,13 @@ ka_preferences_dialog_response (GtkWidget *widget,
                           int response,
                           KaPreferencesDialog *dialog)
 {
-  GError *error = NULL;
-
   if (response != GTK_RESPONSE_HELP) {
       gtk_widget_destroy (widget);
       return;
   }
 
-  gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (dialog->dialog)),
-                "ghelp:krb5-auth-dialog#preferences",
-                gtk_get_current_event_time (), &error);
-
-  if (error) {
-      GtkWidget *message_dialog;
-
-
-      message_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->dialog),
-                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_ERROR,
-                                         GTK_BUTTONS_CLOSE,
-                                         _("There was an error displaying help:\n%s"),
-                                         error->message);
-      gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);
-
-      g_signal_connect (message_dialog, "response",
-                  G_CALLBACK (gtk_widget_destroy),
-                  NULL);
-
-      gtk_widget_show (message_dialog);
-      g_error_free (error);
-  }
+  ka_show_help (gtk_window_get_screen (GTK_WINDOW (dialog->dialog)),
+                "#preferences", GTK_WINDOW(dialog->dialog));
 }
 
 
@@ -607,10 +735,20 @@ ka_preferences_dialog_destroyed (GtkWidget *widget G_GNUC_UNUSED,
 static gboolean
 ka_preferences_dialog_init(KaPreferencesDialog* dialog)
 {
-  dialog->xml = glade_xml_new (KA_DATA_DIR G_DIR_SEPARATOR_S
-                                   PACKAGE "-preferences.glade", NULL, NULL);
+  GError *error = NULL;
+  gboolean ret;
 
-  dialog->dialog = glade_xml_get_widget (dialog->xml, "krb5_auth_dialog_prefs");
+  dialog->xml = gtk_builder_new ();
+
+  ret = gtk_builder_add_from_file(dialog->xml, KA_DATA_DIR G_DIR_SEPARATOR_S
+                                  PACKAGE "-preferences.xml", &error);
+  if (!ret) {
+      g_assert (error);
+      g_assert (error->message);
+      g_error ("Failed to load UI XML: %s", error->message);
+  }
+
+  dialog->dialog = GTK_WIDGET(gtk_builder_get_object (dialog->xml, "krb5_auth_dialog_prefs"));
   g_assert (dialog->dialog);
 
   g_signal_connect (dialog->dialog, "response",
@@ -623,7 +761,10 @@ ka_preferences_dialog_init(KaPreferencesDialog* dialog)
 
   ka_preferences_dialog_setup_principal_entry (dialog);
   ka_preferences_dialog_setup_pkuserid_entry (dialog);
+  ka_preferences_dialog_setup_pkuserid_button (dialog);
+  ka_preferences_dialog_setup_smartcard_toggle (dialog);
   ka_preferences_dialog_setup_pkanchors_entry(dialog);
+  ka_preferences_dialog_setup_pkanchors_button (dialog);
   ka_preferences_dialog_setup_forwardable_toggle (dialog);
   ka_preferences_dialog_setup_proxiable_toggle (dialog);
   ka_preferences_dialog_setup_renewable_toggle (dialog);
@@ -692,6 +833,9 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   bindtextdomain (PACKAGE, LOCALE_DIR);
 
+  gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+                                     DATA_DIR G_DIR_SEPARATOR_S "icons");
+
   ka_preferences_dialog_init(&dialog);
   gtk_main ();
   ka_preferences_dialog_finalize(&dialog);
