@@ -50,12 +50,22 @@ typedef struct _KaAppletPrivate KaAppletPrivate;
 
 GType ka_applet_get_type (void);
 
+/* signals emitted by KaApplet */
+typedef enum {
+  KA_SIGNAL_ACQUIRED_TGT,	/* New TGT acquired */
+  KA_SIGNAL_RENEWED_TGT,	/* TGT got renewed */
+  KA_SIGNAL_EXPIRED_TGT,	/* TGT expired or ticket cache got destroyed */
+  KA_SIGNAL_COUNT
+} KaAppletSignalNumber;
+
 /* public functions */
 gboolean ka_applet_get_show_trayicon(const KaApplet* applet);
 void ka_applet_set_tgt_renewable(KaApplet* applet, gboolean renewable);
 gboolean ka_applet_get_tgt_renewable(const KaApplet* applet);
 guint ka_applet_get_pw_prompt_secs(const KaApplet* applet);
 KaPwDialog* ka_applet_get_pwdialog(const KaApplet* applet);
+void ka_applet_signal_emit(KaApplet* applet, KaAppletSignalNumber signum,
+                           krb5_timestamp expiry);
 
 /* create the applet */
 KaApplet* ka_applet_create(void);
